Package: batchtools
Title: Tools for Computation on Batch Systems
Version: 0.9.15
Authors@R: c(
    person("Michel", "Lang", NULL, "michellang@gmail.com",
      role = c("cre", "aut"), comment = c(ORCID = "0000-0001-9754-0393")),
    person("Bernd", "Bischl", NULL, "bernd_bischl@gmx.net", role = "aut"),
    person("Dirk", "Surmann", NULL, "surmann@statistik.tu-dortmund.de",
      role = "ctb", comment = c(ORCID = "0000-0003-0873-137X"))
    )
Description: As a successor of the packages 'BatchJobs' and 'BatchExperiments',
    this package provides a parallel implementation of the Map function for high
    performance computing systems managed by schedulers 'IBM Spectrum LSF'
    (<https://www.ibm.com/products/hpc-workload-management>),
    'OpenLava' (<https://www.openlava.org/>), 'Univa Grid Engine'/'Oracle Grid
    Engine' (<https://www.univa.com/>), 'Slurm' (<https://slurm.schedmd.com/>),
    'TORQUE/PBS'
    (<https://adaptivecomputing.com/cherry-services/torque-resource-manager/>),
    or 'Docker Swarm' (<https://docs.docker.com/engine/swarm/>).
    A multicore and socket mode allow the parallelization on a local machines,
    and multiple machines can be hooked up via SSH to create a makeshift
    cluster. Moreover, the package provides an abstraction mechanism to define
    large-scale computer experiments in a well-organized and reproducible way.
License: LGPL-3
URL: https://github.com/mllg/batchtools
BugReports: https://github.com/mllg/batchtools/issues
NeedsCompilation: yes
ByteCompile: yes
Encoding: UTF-8
Depends: R (>= 3.0.0)
Imports: backports (>= 1.1.2), base64url (>= 1.1), brew, checkmate (>=
        1.8.5), data.table (>= 1.11.2), digest (>= 0.6.9), fs (>=
        1.2.0), parallel, progress (>= 1.1.1), R6, rappdirs, stats,
        stringi, utils, withr (>= 2.0.0)
Suggests: debugme, doParallel, doMPI, e1071, foreach, future,
        future.batchtools, knitr, parallelMap, ranger, rmarkdown,
        rpart, snow, testthat, tibble
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Packaged: 2021-01-11 08:21:55 UTC; michel
Author: Michel Lang [cre, aut] (<https://orcid.org/0000-0001-9754-0393>),
  Bernd Bischl [aut],
  Dirk Surmann [ctb] (<https://orcid.org/0000-0003-0873-137X>)
Maintainer: Michel Lang <michellang@gmail.com>
Repository: CRAN
Date/Publication: 2021-01-11 12:40:03 UTC
