# Generated by roxygen2: do not edit by hand

S3method(doJobCollection,JobCollection)
S3method(doJobCollection,character)
S3method(execJob,Experiment)
S3method(execJob,Job)
S3method(execJob,JobCollection)
S3method(execJob,character)
S3method(getJob,ExperimentCollection)
S3method(getJob,JobCollection)
S3method(getJobPars,ExperimentRegistry)
S3method(getJobPars,Registry)
S3method(makeJob,ExperimentRegistry)
S3method(makeJob,Registry)
S3method(makeJobCollection,ExperimentRegistry)
S3method(makeJobCollection,Registry)
S3method(print,ClusterFunctions)
S3method(print,ExperimentRegistry)
S3method(print,JobCollection)
S3method(print,Registry)
S3method(print,RuntimeEstimate)
S3method(print,Status)
S3method(print,SubmitJobResult)
S3method(runHook,JobCollection)
S3method(runHook,Registry)
export(Worker)
export(addAlgorithm)
export(addExperiments)
export(addJobTags)
export(addProblem)
export(ajoin)
export(assertRegistry)
export(batchExport)
export(batchMap)
export(batchMapResults)
export(batchReduce)
export(binpack)
export(btlapply)
export(btmapply)
export(cfBrewTemplate)
export(cfHandleUnknownSubmitError)
export(cfKillJob)
export(cfReadBrewTemplate)
export(chunk)
export(clearRegistry)
export(doJobCollection)
export(estimateRuntimes)
export(execJob)
export(findConfFile)
export(findDone)
export(findErrors)
export(findExperiments)
export(findExpired)
export(findJobs)
export(findNotDone)
export(findNotStarted)
export(findNotSubmitted)
export(findOnSystem)
export(findQueued)
export(findRunning)
export(findStarted)
export(findSubmitted)
export(findTagged)
export(findTemplateFile)
export(flatten)
export(getDefaultRegistry)
export(getErrorMessages)
export(getJobNames)
export(getJobPars)
export(getJobResources)
export(getJobStatus)
export(getJobTable)
export(getJobTags)
export(getLog)
export(getStatus)
export(getUsedJobTags)
export(grepLogs)
export(ijoin)
export(killJobs)
export(ljoin)
export(loadRegistry)
export(loadResult)
export(lpt)
export(makeClusterFunctions)
export(makeClusterFunctionsDocker)
export(makeClusterFunctionsInteractive)
export(makeClusterFunctionsLSF)
export(makeClusterFunctionsMulticore)
export(makeClusterFunctionsOpenLava)
export(makeClusterFunctionsSGE)
export(makeClusterFunctionsSSH)
export(makeClusterFunctionsSlurm)
export(makeClusterFunctionsSocket)
export(makeClusterFunctionsTORQUE)
export(makeExperimentRegistry)
export(makeJob)
export(makeJobCollection)
export(makeRegistry)
export(makeSubmitJobResult)
export(ojoin)
export(reduceResults)
export(reduceResultsDataTable)
export(reduceResultsList)
export(removeAlgorithms)
export(removeExperiments)
export(removeJobTags)
export(removeProblems)
export(removeRegistry)
export(resetJobs)
export(rjoin)
export(runHook)
export(runOSCommand)
export(saveRegistry)
export(setDefaultRegistry)
export(setJobNames)
export(showLog)
export(sjoin)
export(submitJobs)
export(summarizeExperiments)
export(sweepRegistry)
export(syncRegistry)
export(testJob)
export(ujoin)
export(unwrap)
export(waitForJobs)
import(checkmate)
import(data.table)
import(stringi)
import(utils)
importFrom(R6,R6Class)
importFrom(base64url,base32_decode)
importFrom(base64url,base32_encode)
importFrom(brew,brew)
importFrom(digest,digest)
importFrom(progress,progress_bar)
importFrom(rappdirs,site_config_dir)
importFrom(rappdirs,user_config_dir)
importFrom(stats,pexp)
importFrom(stats,predict)
importFrom(stats,runif)
importFrom(withr,local_dir)
importFrom(withr,local_options)
importFrom(withr,with_dir)
importFrom(withr,with_seed)
useDynLib(batchtools,c_binpack)
useDynLib(batchtools,c_lpt)
useDynLib(batchtools,count_not_missing)
useDynLib(batchtools,fill_gaps)
