% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{package}
\name{batchtools-package}
\alias{batchtools}
\alias{batchtools-package}
\title{batchtools: Tools for Computation on Batch Systems}
\description{
For bug reports and feature requests please use the tracker:
\url{https://github.com/mlr-org/batchtools}.
}
\section{Package options}{

\describe{
  \item{\code{batchtools.verbose}}{
    Verbosity. Set to \code{FALSE} to suppress info messages and progress bars.
  }
  \item{\code{batchtools.progress}}{
    Progress bars. Set to \code{FALSE} to disable them.
  }
  \item{\code{batchtools.timestamps}}{
    Add time stamps to log output. Set to \code{FALSE} to disable them.
  }
}
Furthermore, you may enable a debug mode using the \pkg{debugme} package by
setting the environment variable \dQuote{DEBUGME} to \dQuote{batchtools} before
loading \pkg{batchtools}.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/mlr-org/batchtools}
  \item \url{https://batchtools.mlr-org.com}
  \item Report bugs at \url{https://github.com/mlr-org/batchtools/issues}
}

}
\author{
\strong{Maintainer}: Michel Lang \email{michellang@gmail.com} (\href{https://orcid.org/0000-0001-9754-0393}{ORCID})

Authors:
\itemize{
  \item Bernd Bischl \email{bernd_bischl@gmx.net}
}

Other contributors:
\itemize{
  \item Dirk Surmann \email{surmann@statistik.tu-dortmund.de} (\href{https://orcid.org/0000-0003-0873-137X}{ORCID}) [contributor]
}

}
